==> wowit-0.4.22.tar.gz: lib/widgets/base_widget.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:get/get.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/controllers/connect_controller.dart';
import 'package:wowit/common.dart';
import 'package:wowit/widgets/dialog/dialog_supper_admin.dart';

abstract class BaseWidget<T extends BaseController> extends StatelessWidget {
  const BaseWidget(
    T baseController, {
    super.key,
    this.isComponent = true,
    this.tag,
    this.global = true,
  }) : _baseController = baseController;

  final bool global;
  final bool isComponent;
  final String? tag;
  final T _baseController;

  T getController({String? tag}) {
    if (global) {
      if (Get.isRegistered<T>(tag: tag) || Get.isPrepared<T>(tag: tag)) {
        return Get.find<T>(tag: tag);
      } else {
        return Get.put<T>(_baseController, tag: tag);
      }
    } else {
      return _baseController;
    }
  }

  T get controller => getController(tag: tag);

  @override
  String toString({DiagnosticLevel minLevel = DiagnosticLevel.info}) {
    return runtimeType.toString();
  }

  @override
  Widget build(BuildContext context) {
    return _getWidget(context);
  }

  void initState(GetBuilderState<T> state) {
    Get.log(
      '${toString()} initState',
      isError: false,
    );
    if (Common.supperAdmin && !isComponent) {
      Common.messageIdBackUp = {};
    }
    MyConnectivity.instance.initialise(_baseController);
    if (Get.isPrepared<T>(tag: tag)) {
      Get.replace<T>(_baseController, tag: tag);
    } else {
      Get.lazyPut(() => _baseController);
    }
    if (Common.supperAdmin && isComponent) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        _baseController.messageIdBackUp.addAll(Common.messageIdBackUp);
      });
    }
  }

  void dispose(GetBuilderState<T> state) {
    Get.log(
      '${toString()} dispose',
      isError: false,
    );
    if (!global) {
      state.controller = null;
    }
  }

  void didUpdateWidget(GetBuilder oldWidget, GetBuilderState<T> state) {
    Get.log(
      '${toString()} didUpdateWidget',
      isError: false,
    );
  }

  void didChangeDependencies(GetBuilderState<T> state) {
    Get.log(
      '${toString()} didChangeDependencies',
      isError: false,
    );
  }

  Widget _getWidget(BuildContext context) {
    return GetBuilder<T>(
      global: global,
      init: _baseController,
      tag: tag,
      initState: initState,
      dispose: dispose,
      didUpdateWidget: didUpdateWidget,
      didChangeDependencies: didChangeDependencies,
      builder: (controller) {
        Get.log(
          '${toString()} start with Controller: ${controller.runtimeType}',
          isError: false,
        );
        var cachedData = controller.getData();
        if (cachedData.isEmpty) {
          return FutureBuilder(
            future: controller.loadData(),
            builder: (
              context,
              AsyncSnapshot<Map<String, List<Map<String, Object>>>?> snapshot,
            ) {
              Get.log(
                '${toString()} rebuild new data',
                isError: false,
              );
              if (snapshot.connectionState == ConnectionState.waiting) {
                Get.log(
                  '${toString()} ConnectionState.waiting return getSkeletonsView',
                  isError: false,
                );
                return getSkeletonView(context, controller);
              } else if (snapshot.hasError) {
                Get.log(
                  '${toString()} has error return getErrorView message: ${snapshot.error}',
                  isError: true,
                );
                return _errorViewAndNoInterNet(snapshot, controller);
              } else {
                Map<String, List<Map<String, Object>>>? data = snapshot.data;
                if (data != null && data.isEmpty) {
                  Get.log(
                    '${toString()} data is empty return getEmptyView',
                    isError: false,
                  );
                  return _emptyViewAndNoInterNet(controller);
                }
                data ??= {};
                controller.setData(data);
                Get.log(
                  '${toString()} done process return getMainView',
                  isError: false,
                );
                if (Common.supperAdmin && !isComponent) {
                  return _widgetShowMessageID(
                    context,
                    getMainView(context, data, controller),
                    controller,
                  );
                }
                return getMainView(context, data, controller);
              }
            },
          );
        } else {
          Get.log(
            '${toString()} rebuild old data',
            isError: false,
          );
          return getMainView(context, cachedData, controller);
        }
      },
    );
  }

  Widget getWaitingView(BuildContext context) {
    return Common.loadingScreen;
  }

  Widget getSkeletonView(BuildContext context, T controller) {
    String skeleton = isComponent ? 'component' : 'view';
    Get.log(
      '${toString()} build skeleton $skeleton',
      isError: false,
    );
    //show waiting view when is not component
    if (!isComponent) {
      return getWaitingView(context);
    }
    return Material(
      child: SafeArea(
        child: LayoutBuilder(
          builder: (context, constraints) {
            double heightItem = 150;
            final double height = constraints.maxHeight != double.infinity
                ? constraints.maxHeight
                : heightItem;
            final double width = constraints.maxWidth;
            if (heightItem > height) {
              heightItem = height;
            }
            final int count = height ~/ heightItem;
            return ListView.separated(
              shrinkWrap: true,
              padding: const EdgeInsets.all(16),
              physics: const NeverScrollableScrollPhysics(),
              separatorBuilder: (_, __) => const SizedBox(height: 16),
              itemCount: count,
              itemBuilder: (_, __) => SizedBox(
                height: heightItem,
                width: width,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Expanded(
                      child: Row(
                        children: [
                          Expanded(
                            flex: 2,
                            child: Container(
                              decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(10),
                                color: Colors.grey[100],
                              ),
                            ),
                          ),
                          const SizedBox(width: 8),
                          Expanded(
                            flex: 1,
                            child: Column(
                              children: [
                                Expanded(
                                  child: Container(
                                    decoration: BoxDecoration(
                                      borderRadius: BorderRadius.circular(10),
                                      color: Colors.grey[100],
                                    ),
                                  ),
                                ),
                                const SizedBox(height: 8),
                                Expanded(
                                  child: Container(
                                    decoration: BoxDecoration(
                                      borderRadius: BorderRadius.circular(10),
                                      color: Colors.grey[100],
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          )
                        ],
                      ),
                    ),
                    const SizedBox(height: 8),
                    Container(
                      height: 20,
                      width: MediaQuery.of(context).size.width,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100],
                      ),
                    ),
                    const SizedBox(height: 8),
                    Container(
                      height: 20,
                      width: MediaQuery.of(context).size.width / 1.5,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100],
                      ),
                    )
                  ],
                ),
              ),
            );
          },
        ),
      ),
    );
  }

  Widget _errorViewAndNoInterNet(
    AsyncSnapshot<Map<String, List<Map<String, Object>>>?> snapshot,
    T controller,
  ) {
    if (controller.isOnline) {
      controller.loadData();
      return getErrorView(snapshot, controller);
    } else {
      return noInternet();
    }
  }

  Widget getErrorView(
    AsyncSnapshot<Map<String, List<Map<String, Object>>>?> snapshot,
    T controller,
  ) {
    return Center(
      child: Image.asset(
        'assets/images/404-error.png',
        height: 100,
        width: 100,
      ),
    );
  }

  Widget _emptyViewAndNoInterNet(T controller) {
    if (controller.isOnline) {
      controller.loadData();
      return getEmptyView(controller);
    } else {
      return noInternet();
    }
  }

  Widget getEmptyView(T baseController) {
    return Center(
      child: Image.asset(
        'assets/images/emptyFolder.png',
        height: 100,
        width: 100,
      ),
    );
  }

  Widget noInternet() {
    Get.log('$runtimeType no internet', isError: true);
    return Common.noInternetScreen;
  }

  Widget _widgetShowMessageID(
    BuildContext context,
    Widget mainView,
    T baseController,
  ) {
    return Stack(
      children: [
        mainView,
        Positioned(
          right: 10,
          bottom: 50,
          child: Material(
            borderRadius: BorderRadius.circular(100),
            child: InkWell(
              onTap: () {
                // Common.showKeyMessage = !Common.showKeyMessage;
                // baseController.update();
                showDialog(context, baseController);
              },
              child: Container(
                width: 50,
                height: 50,
                padding: const EdgeInsets.all(8),
                decoration: BoxDecoration(
                  color: Colors.blue[50],
                  shape: BoxShape.circle,
                ),
                child: Center(
                  child: !Common.showKeyMessage
                      ? const Icon(Icons.visibility_outlined)
                      : const Icon(Icons.visibility_off_outlined),
                ),
              ),
            ),
          ),
        )
      ],
    );
  }

  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    T controller,
  );

  showDialog(BuildContext context, T controller) {
    Get.dialog(
      DiaLogSupperAdmin(
        controller: controller,
      ),
    );
  }
}

==> wowit-0.4.27.tar.gz: lib/widgets/base_widget.dart <==
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:get/get.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/controllers/connect_controller.dart';
import 'package:wowit/common.dart';
import 'package:wowit/widgets/dialog/dialog_supper_admin.dart';

abstract class BaseWidget<T extends BaseController> extends StatelessWidget {
  const BaseWidget(
    T baseController, {
    super.key,
    this.isComponent = true,
    this.tag,
    this.global = true,
  }) : _baseController = baseController;

  final bool global;
  final bool isComponent;
  final String? tag;
  final T _baseController;

  T getController({String? tag}) {
    if (global) {
      if (Get.isRegistered<T>(tag: tag) || Get.isPrepared<T>(tag: tag)) {
        return Get.find<T>(tag: tag);
      } else {
        return Get.put<T>(_baseController, tag: tag);
      }
    } else {
      return _baseController;
    }
  }

  T get controller => getController(tag: tag);

  @override
  String toString({DiagnosticLevel minLevel = DiagnosticLevel.info}) {
    return runtimeType.toString();
  }

  @override
  Widget build(BuildContext context) {
    return _getWidget(context);
  }

  void initState(GetBuilderState<T> state) {
    Get.log(
      '${toString()} initState',
      isError: false,
    );
    if (Common.supperAdmin && !isComponent) {
      Common.messageIdBackUp = {};
    }
    MyConnectivity.instance.initialise(controller);
    if (Get.isPrepared<T>(tag: tag)) {
      Get.replace<T>(_baseController, tag: tag);
    } else {
      Get.lazyPut(() => _baseController);
    }
    if (Common.supperAdmin && isComponent) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        _baseController.messageIdBackUp.addAll(Common.messageIdBackUp);
      });
    }
  }

  void dispose(GetBuilderState<T> state) {
    Get.log(
      '${toString()} dispose',
      isError: false,
    );
    if (!global) {
      state.controller = null;
    }
  }

  void didUpdateWidget(GetBuilder oldWidget, GetBuilderState<T> state) {
    Get.log(
      '${toString()} didUpdateWidget',
      isError: false,
    );
  }

  void didChangeDependencies(GetBuilderState<T> state) {
    Get.log(
      '${toString()} didChangeDependencies',
      isError: false,
    );
  }

  Widget _getWidget(BuildContext context) {
    return GetBuilder<T>(
      global: global,
      init: _baseController,
      tag: tag,
      initState: initState,
      dispose: dispose,
      didUpdateWidget: didUpdateWidget,
      didChangeDependencies: didChangeDependencies,
      builder: (controller) {
        Get.log(
          '${toString()} start with Controller: ${controller.runtimeType}',
          isError: false,
        );
        var cachedData = controller.getData();
        if (cachedData.isEmpty) {
          return FutureBuilder(
            future: controller.loadData(),
            builder: (
              context,
              AsyncSnapshot<Map<String, List<Map<String, Object>>>?> snapshot,
            ) {
              Get.log(
                '${toString()} rebuild new data',
                isError: false,
              );
              if (snapshot.connectionState == ConnectionState.waiting) {
                Get.log(
                  '${toString()} ConnectionState.waiting return getSkeletonsView',
                  isError: false,
                );
                return getSkeletonView(context, controller);
              } else if (snapshot.hasError) {
                Get.log(
                  '${toString()} has error return getErrorView message: ${snapshot.error}',
                  isError: true,
                );
                return _errorViewAndNoInterNet(snapshot, controller);
              } else {
                Map<String, List<Map<String, Object>>>? data = snapshot.data;
                if (data != null && data.isEmpty) {
                  Get.log(
                    '${toString()} data is empty return getEmptyView',
                    isError: false,
                  );
                  return _emptyViewAndNoInterNet(controller);
                }
                data ??= {};
                controller.setData(data);
                Get.log(
                  '${toString()} done process return getMainView',
                  isError: false,
                );
                if (Common.supperAdmin && !isComponent) {
                  return _widgetShowMessageID(
                    context,
                    getMainView(context, data, controller),
                    controller,
                  );
                }
                return getMainView(context, data, controller);
              }
            },
          );
        } else {
          Get.log(
            '${toString()} rebuild old data',
            isError: false,
          );
          return getMainView(context, cachedData, controller);
        }
      },
    );
  }

  Widget getWaitingView(BuildContext context) {
    return Common.loadingScreen;
  }

  Widget getSkeletonView(BuildContext context, T controller) {
    String skeleton = isComponent ? 'component' : 'view';
    Get.log(
      '${toString()} build skeleton $skeleton',
      isError: false,
    );
    //show waiting view when is not component
    if (!isComponent) {
      return getWaitingView(context);
    }
    return Material(
      child: SafeArea(
        child: LayoutBuilder(
          builder: (context, constraints) {
            double heightItem = 150;
            final double height = constraints.maxHeight != double.infinity
                ? constraints.maxHeight
                : heightItem;
            final double width = constraints.maxWidth;
            if (heightItem > height) {
              heightItem = height;
            }
            final int count = height ~/ heightItem;
            return ListView.separated(
              shrinkWrap: true,
              padding: const EdgeInsets.all(16),
              physics: const NeverScrollableScrollPhysics(),
              separatorBuilder: (_, __) => const SizedBox(height: 16),
              itemCount: count,
              itemBuilder: (_, __) => SizedBox(
                height: heightItem,
                width: width,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Expanded(
                      child: Row(
                        children: [
                          Expanded(
                            flex: 2,
                            child: Container(
                              decoration: BoxDecoration(
                                borderRadius: BorderRadius.circular(10),
                                color: Colors.grey[100],
                              ),
                            ),
                          ),
                          const SizedBox(width: 8),
                          Expanded(
                            flex: 1,
                            child: Column(
                              children: [
                                Expanded(
                                  child: Container(
                                    decoration: BoxDecoration(
                                      borderRadius: BorderRadius.circular(10),
                                      color: Colors.grey[100],
                                    ),
                                  ),
                                ),
                                const SizedBox(height: 8),
                                Expanded(
                                  child: Container(
                                    decoration: BoxDecoration(
                                      borderRadius: BorderRadius.circular(10),
                                      color: Colors.grey[100],
                                    ),
                                  ),
                                ),
                              ],
                            ),
                          )
                        ],
                      ),
                    ),
                    const SizedBox(height: 8),
                    Container(
                      height: 20,
                      width: MediaQuery.of(context).size.width,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100],
                      ),
                    ),
                    const SizedBox(height: 8),
                    Container(
                      height: 20,
                      width: MediaQuery.of(context).size.width / 1.5,
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100],
                      ),
                    )
                  ],
                ),
              ),
            );
          },
        ),
      ),
    );
  }

  Widget _errorViewAndNoInterNet(
    AsyncSnapshot<Map<String, List<Map<String, Object>>>?> snapshot,
    T controller,
  ) {
    if (controller.isOnline) {
      controller.loadData();
      return getErrorView(snapshot, controller);
    } else {
      return noInternet();
    }
  }

  Widget getErrorView(
    AsyncSnapshot<Map<String, List<Map<String, Object>>>?> snapshot,
    T controller,
  ) {
    return Center(
      child: Image.asset(
        'assets/images/404-error.png',
        height: 100,
        width: 100,
      ),
    );
  }

  Widget _emptyViewAndNoInterNet(T controller) {
    if (controller.isOnline) {
      controller.loadData();
      return getEmptyView(controller);
    } else {
      return noInternet();
    }
  }

  Widget getEmptyView(T baseController) {
    return Center(
      child: Image.asset(
        'assets/images/emptyFolder.png',
        height: 100,
        width: 100,
      ),
    );
  }

  Widget noInternet() {
    Get.log('$runtimeType no internet', isError: true);
    return Common.noInternetScreen;
  }

  Widget _widgetShowMessageID(
    BuildContext context,
    Widget mainView,
    T baseController,
  ) {
    return Stack(
      children: [
        mainView,
        Positioned(
          right: 10,
          bottom: 50,
          child: Material(
            borderRadius: BorderRadius.circular(100),
            child: InkWell(
              onTap: () {
                // Common.showKeyMessage = !Common.showKeyMessage;
                // baseController.update();
                showDialog(context, baseController);
              },
              child: Container(
                width: 50,
                height: 50,
                padding: const EdgeInsets.all(8),
                decoration: BoxDecoration(
                  color: Colors.blue[50],
                  shape: BoxShape.circle,
                ),
                child: Center(
                  child: !Common.showKeyMessage
                      ? const Icon(Icons.visibility_outlined)
                      : const Icon(Icons.visibility_off_outlined),
                ),
              ),
            ),
          ),
        )
      ],
    );
  }

  Widget getMainView(
    BuildContext context,
    Map<String, List<Map<String, Object>>> data,
    T controller,
  );

  showDialog(BuildContext context, T controller) {
    Get.dialog(
      DiaLogSupperAdmin(
        controller: controller,
      ),
    );
  }
}
